// Copyright 2014-2016 Insight Software Consortium.
// Copyright 2004-2008 Roman Yakovenko.
// Distributed under the Boost Software License, Version 1.0.
// See http://www.boost.org/LICENSE_1_0.txt

#ifndef __unnamed_enums_bug2_hpp__
#define __unnamed_enums_bug2_hpp__

enum{ z1, z2 };

#endif//__unnamed_enums_bug2_hpp__
